/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "watools.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <dirent.h>

char* WATools::ap( const char* ifname ) ///returns current AP (for the specified interface). Request SIOCGIWAP
{
        int socket = iw_sockets_open(); //get kernel socket
        if (socket<0)
                return 0;

        struct iwreq req;
        if( iw_get_ext(socket, ifname, SIOCGIWAP, &req) < 0 ) { // try to get ap a7dress.
                iw_sockets_close( socket );
                return 0; // no ap address
        } else {
                static char buffer[64];
                iw_ether_ntop((const struct ether_addr *)req.u.ap_addr.sa_data, buffer);
                iw_sockets_close( socket );
                return buffer;
        }
}

int WATools::quality( const char* ifname ) ///returns active link quality in range 0-100 (for the specified interface)
{
        int has_range;
        iw_range range;
        iw_statistics stats;

        int socket = iw_sockets_open(); //get kernel socket
        if (socket<0)
                return 0;

        if ( iw_get_range_info(socket, ifname, &range) < 0 )
                has_range = 0;
        else
                has_range = 1;

        iw_get_stats(socket, ifname, &stats, &range, has_range);
        iw_sockets_close( socket );

        int std_qual = (100*stats.qual.qual)/range.max_qual.qual; //calculate quality in range 0-100.
        return std_qual;
}

int WATools::txpower( const char* ifname ) ///returns current txpower in mW (for the specified interface). Request SIOCGIWTXPOW
{
        int socket = iw_sockets_open(); //get kernel socket
        if (socket<0)
                return 0;

        struct iwreq req;
        if( iw_get_ext(socket, ifname, SIOCGIWTXPOW, &req) < 0 ) { // try to get txpower.
                iw_sockets_close( socket );
                return 0; // no txpower
        } else {
                iw_sockets_close( socket );
                if (req.u.txpower.disabled)
                        return -1;
                else
                        return req.u.txpower.value;
        }
}

bool WATools::isWireless( const char* ifname ) /// returns TRUE if the specified interface supports Wireless Extensions
{
        int socket = iw_sockets_open(); //get kernel socket
        if (socket<0)
                return 0;

        struct iwreq req;
        if( iw_get_ext(socket, ifname, SIOCGIWNAME, &req) < 0 ) { // try to get wireless name.
                iw_sockets_close( socket );
                return false; // no wireless name -> no wireless extensions
        } else {
                iw_sockets_close( socket );
                return true;
        }
}

char* WATools::devices() /// returns comma separated list of wireless interfaces available on the system.
{
        DIR* dir;
        struct dirent* entry;
        char* name;
        static char devs[64];
        memset( devs, 0, sizeof(devs) );

        dir = opendir("/sys/class/net");
        while ( (entry = readdir(dir)) != NULL )
        {
                name = entry->d_name;
                if ( memcmp(name, ".", 1)==0 || !isWireless(name) ) // ignore '.', '..' and devices w/out wireless extensions.
                        continue;
                else {
                        if ( strlen(devs)==0 )
                                strcpy(devs, name);
                        else {
                                strcat( devs, "," );
                                strcat( devs, name );
                        }
                }
        }
        return devs;
}
